@@ -2,8 +2,8 @@ import sys
 import typing
 import asyncio
 import logging
+import warnings
 import traceback
-import copy
 
 from dico import (
     ApplicationCommand,
@@ -20,6 +20,8 @@ from .command import autocomplete as autocomplete_deco
 from .deco import command as command_deco
 from .component import ComponentCallback
 from .context import InteractionContext
+from .exception import AlreadyExists, NotExists
+from .modal import ModalCallback
 
 
 class InteractionClient:
@@ -52,6 +54,7 @@ class InteractionClient:
                  client: typing.Optional[Client] = None,
                  auto_register_commands: bool = False,
                  guild_id_lock: typing.Optional[Snowflake.TYPING] = None,
+                 guild_ids_lock: typing.Optional[typing.List[Snowflake.TYPING]] = None,
                  context_cls: typing.Type[InteractionContext] = InteractionContext):
         self.loop = loop or asyncio.get_event_loop()
 
@@ -62,10 +65,16 @@ class InteractionClient:
 
         self.components = {}
         self.autocompletes = {}
+        self.modals = {}
 
         self.logger = logging.getLogger("dico.interaction")
         self.respond_via_endpoint = respond_via_endpoint
-        self.guild_id_lock = guild_id_lock
+        if guild_id_lock and guild_ids_lock:
+            raise ValueError("You can't set both guild_id and guild_ids")
+        elif guild_id_lock:
+            warnings.warn("guild_id_lock is deprecated, use guild_ids_lock instead.", DeprecationWarning, stacklevel=2)
+            guild_ids_lock = [guild_id_lock]
+        self.guild_id_locks = guild_ids_lock
         self.context_cls = context_cls
         self.client = client
         if self.client is not None:
@@ -119,6 +128,12 @@ class InteractionClient:
                     target = self.components.get(maybe[0])
         elif interaction.type.application_command_autocomplete:
             target = self.get_autocomplete(interaction)
+        elif interaction.type.modal_submit:
+            target = self.modals.get(interaction.data.custom_id)
+            if not target:
+                maybe = [x for x in self.modals if interaction.data.custom_id.startswith(x)]
+                if maybe:
+                    target = self.modals.get(maybe[0])
         else:
             return
 
@@ -264,10 +279,11 @@ class InteractionClient:
         cmds = {"global": [], "guild": {}}
 
         for cmd in self.commands.values():
-            if cmd.guild_id is not None:
-                if cmds["guild"].get(cmd.guild_id) is None:
-                    cmds["guild"][cmd.guild_id] = []
-                cmds["guild"][cmd.guild_id].append(cmd.command)
+            if cmd.guild_ids:
+                for guild_id in cmd.guild_ids:
+                    if cmds["guild"].get(guild_id) is None:
+                        cmds["guild"][guild_id] = []
+                    cmds["guild"][guild_id].append(cmd.command)
             else:
                 cmds["global"].append(cmd.command)
 
@@ -275,49 +291,61 @@ class InteractionClient:
 
         for p_cmd in self.subcommands.values():
             for c_cmd in p_cmd.values():
-                if c_cmd.guild_id is not None:
-                    subcommands["guild"][c_cmd.guild_id] = {}
+                if c_cmd.guild_ids:
+                    for guild_id in c_cmd.guild_ids:
+                        subcommands["guild"][guild_id] = {}
 
         for p_cmd in self.subcommand_groups.values():
             for c_cmd in p_cmd.values():
                 for s_cmd in c_cmd.values():
-                    if s_cmd.guild_id is not None:
-                        subcommands["guild"][s_cmd.guild_id] = {}
+                    if s_cmd.guild_ids is not None:
+                        for guild_id in s_cmd.guild_ids:
+                            subcommands["guild"][guild_id] = {}
 
         for p_k, p_v in self.subcommands.items():
             for c_k, c_v in p_v.items():
-                if c_v.guild_id is not None:
-                    data = subcommands["guild"][c_v.guild_id]
+                datas = []
+                if c_v.guild_ids:
+                    for guild_id in c_v.guild_ids:
+                        datas.append(subcommands["guild"][guild_id])
                 else:
-                    data = subcommands["global"]
+                    datas = [subcommands["global"]]
 
-                if data.get(p_k) is None:
-                    data[p_k] = {}
-                data[p_k][c_k] = c_v.command
+                for data in datas:
+                    if data.get(p_k) is None:
+                        data[p_k] = {}
+                    data[p_k][c_k] = c_v.command
 
-                if c_v.guild_id is not None:
-                    subcommands["guild"][c_v.guild_id] = data
-                else:
-                    subcommands["global"] = data
+                for data in datas:
+                    if c_v.guild_ids:
+                        for guild_id in c_v.guild_ids:
+                            subcommands["guild"][guild_id] = data
+                    else:
+                        subcommands["global"] = data
 
         for p_k, p_v in self.subcommand_groups.items():
             for c_k, c_v in p_v.items():
                 for s_k, s_v in c_v.items():
-                    if s_v.guild_id is not None:
-                        data = subcommands["guild"][s_v.guild_id]
+                    datas = []
+                    if s_v.guild_ids:
+                        for guild_id in s_v.guild_ids:
+                            datas.append(subcommands["guild"][guild_id])
                     else:
-                        data = subcommands["global"]
+                        datas = [subcommands["global"]]
 
-                    if data.get(p_k) is None:
-                        data[p_k] = {}
-                    if data[p_k].get(c_k) is None:
-                        data[p_k][c_k] = {}
-                    data[p_k][c_k][s_k] = s_v.command
+                    for data in datas:
+                        if data.get(p_k) is None:
+                            data[p_k] = {}
+                        if data[p_k].get(c_k) is None:
+                            data[p_k][c_k] = {}
+                        data[p_k][c_k][s_k] = s_v.command
 
-                    if s_v.guild_id is not None:
-                        subcommands["guild"][s_v.guild_id] = data
-                    else:
-                        subcommands["global"] = data
+                    for data in datas:
+                        if s_v.guild_ids is not None:
+                            for guild_id in s_v.guild_ids:
+                                subcommands["guild"][guild_id] = data
+                        else:
+                            subcommands["global"] = data
 
         def get_command(data):
             data = data.values()
@@ -360,8 +388,8 @@ class InteractionClient:
 
         :param interaction: Command to add.
         """
-        if self.guild_id_lock:
-            interaction.guild_id = self.guild_id_lock
+        if self.guild_id_locks:
+            interaction.guild_ids = self.guild_id_locks
         subcommand_group = interaction.subcommand_group
         subcommand = interaction.subcommand
         name = interaction.command.name
@@ -371,17 +399,17 @@ class InteractionClient:
             if subcommand_group not in self.subcommand_groups[name]:
                 self.subcommand_groups[name][subcommand_group] = {}
             if subcommand in self.subcommand_groups[name][subcommand_group]:
-                raise
+                raise AlreadyExists("command", f"{subcommand_group} {subcommand} {name}")
             self.subcommand_groups[name][subcommand_group][subcommand] = interaction
         elif subcommand:
             if name not in self.subcommands:
                 self.subcommands[name] = {}
             if subcommand in self.subcommands[name]:
-                raise
+                raise AlreadyExists("command", f"{subcommand} {name}")
             self.subcommands[name][subcommand] = interaction
         else:
             if name in self.commands:
-                raise
+                raise AlreadyExists("command", name)
             self.commands[name] = interaction
 
     def remove_command(self, interaction: InteractionCommand):
@@ -398,38 +426,40 @@ class InteractionClient:
                     subcommand in self.subcommand_groups[name][subcommand_group]:
                 del self.subcommand_groups[name][subcommand_group][subcommand]
             else:
-                raise
+                raise NotExists("command", f"{subcommand_group} {subcommand} {name}")
         elif subcommand:
             if name in self.subcommands and subcommand in self.subcommands[name]:
                 del self.subcommands[name][subcommand]
             else:
-                raise
+                raise NotExists("command", f"{subcommand} {name}")
         else:
             if name in self.commands:
                 del self.commands[name]
             else:
-                raise
+                raise NotExists("command", name)
 
-    def add_callback(self, callback: ComponentCallback):
+    def add_callback(self, callback: typing.Union[ComponentCallback, ModalCallback]):
         """
-        Adds component callback to the client.
+        Adds component or modal callback to the client.
 
         :param callback: Callback to add.
         """
-        if callback.custom_id in self.components:
-            raise
-        self.components[callback.custom_id] = callback
+        tgt = self.modals if isinstance(callback, ModalCallback) else self.components
+        if callback.custom_id in tgt:
+            raise AlreadyExists(f"{'modal' if isinstance(callback, ModalCallback) else 'component'} callback", callback.custom_id)
+        tgt[callback.custom_id] = callback
 
-    def remove_callback(self, callback: ComponentCallback):
+    def remove_callback(self, callback: typing.Union[ComponentCallback, ModalCallback]):
         """
         Removes callback from client.
 
         :param callback: Callback to remove.
         """
-        if callback.custom_id in self.components:
-            del self.components[callback.custom_id]
+        tgt = self.modals if isinstance(callback, ModalCallback) else self.components
+        if callback.custom_id in tgt:
+            del tgt[callback.custom_id]
         else:
-            raise
+            raise NotExists(f"{'modal' if isinstance(callback, ModalCallback) else 'component'} callback", callback.custom_id)
 
     def add_autocomplete(self, autocomplete: AutoComplete):
         """
@@ -442,9 +472,9 @@ class InteractionClient:
         elif autocomplete.subcommand:
             key = f"{autocomplete.name}:{autocomplete.subcommand}:{autocomplete.option}"
         else:
-            key = f"{autocomplete.name}:{autocomplete.name}"
+            key = f"{autocomplete.name}:{autocomplete.option}"
         if key in self.autocompletes:
-            raise
+            raise AlreadyExists("autocomplete", f"/{key.replace(':', ' ')}:")
         self.autocompletes[key] = autocomplete
 
     def remove_autocomplete(self, autocomplete: AutoComplete):
@@ -458,9 +488,9 @@ class InteractionClient:
         elif autocomplete.subcommand:
             key = f"{autocomplete.name}:{autocomplete.subcommand}:{autocomplete.option}"
         else:
-            key = f"{autocomplete.name}:{autocomplete.name}"
+            key = f"{autocomplete.name}:{autocomplete.option}"
         if key not in self.autocompletes:
-            raise
+            raise NotExists("autocomplete", f"/{key.replace(':', ' ')}:")
         del self.autocompletes[key]
 
     def command(self,
@@ -474,7 +504,8 @@ class InteractionClient:
                 command_type: typing.Union[int, ApplicationCommandTypes] = ApplicationCommandTypes.CHAT_INPUT,
                 options: typing.List[ApplicationCommandOption] = None,
                 default_permission: bool = True,
-                guild_id: typing.Union[int, str, Snowflake] = None,
+                guild_id: Snowflake.TYPING = None,
+                guild_ids: typing.List[Snowflake.TYPING] = None,
                 connector: typing.Dict[str, str] = None):
         """
         Creates and registers interaction command to the client.
@@ -494,7 +525,8 @@ class InteractionClient:
         :param command_type: Type of command.
         :param options: Options of the command.
         :param default_permission: Whether default permission is enabled.
-        :param guild_id: ID of the guild.
+        :param guild_id: ID of the guild. This is deprecated since ``0.0.7``.
+        :param guild_ids: List of ID of the guilds.
         :param connector: Option parameter connector.
         """
         def wrap(coro):
@@ -508,6 +540,7 @@ class InteractionClient:
                                options=options,
                                default_permission=default_permission,
                                guild_id=guild_id,
+                               guild_ids=guild_ids,
                                connector=connector)(coro)
             self.add_command(cmd)
             return cmd
@@ -523,7 +556,8 @@ class InteractionClient:
               subcommand_group_description: str = None,
               options: typing.List[ApplicationCommandOption] = None,
               default_permission: bool = True,
-              guild_id: typing.Union[int, str, Snowflake] = None,
+              guild_id: Snowflake.TYPING = None,
+              guild_ids: typing.List[Snowflake.TYPING] = None,
               connector: typing.Dict[str, str] = None):
         """
         Creates and registers slash command to the client.
@@ -554,7 +588,8 @@ class InteractionClient:
         :param subcommand_group_description: Description of subcommand group.
         :param options: Options of the command.
         :param default_permission: Whether default permission is enabled.
-        :param guild_id: ID of the guild.
+        :param guild_id: ID of the guild. This is deprecated since ``0.0.7``.
+        :param guild_ids: List of ID of the guilds.
         :param connector: Option parameter connector.
         """
         return self.command(name=name,
@@ -566,22 +601,25 @@ class InteractionClient:
                             options=options,
                             default_permission=default_permission,
                             guild_id=guild_id,
+                            guild_ids=guild_ids,
                             connector=connector)
 
     def context_menu(self,
                      name: str = None,
                      menu_type: typing.Union[int, ApplicationCommandTypes] = ApplicationCommandTypes.MESSAGE,
-                     guild_id: typing.Union[int, str, Snowflake] = None):
+                     guild_id: Snowflake.TYPING = None,
+                     guild_ids: typing.List[Snowflake.TYPING] = None):
         """
         Creates and registers context menu to the client.
 
         :param name: Name of the command.
         :param menu_type: Type of the context menu.
-        :param guild_id: ID of the guild.
+        :param guild_id: ID of the guild. This is deprecated since ``0.0.7``.
+        :param guild_ids: List of ID of the guilds.
         """
         if int(menu_type) == ApplicationCommandTypes.CHAT_INPUT:
             raise TypeError("unsupported context menu type for context_menu decorator.")
-        return self.command(name=name, description="", command_type=menu_type, guild_id=guild_id)
+        return self.command(name=name, description="", command_type=menu_type, guild_id=guild_id, guild_ids=guild_ids)
 
     def component_callback(self, custom_id: str = None):
         """
@@ -619,3 +657,15 @@ class InteractionClient:
             self.add_autocomplete(autocomplete)
             return autocomplete
         return wrap
+
+    def modal(self, custom_id: str = None):
+        """
+        Adds modal callback to the client.
+
+        :param custom_id: Custom ID of the modal. Can be prefix of the custom ID.
+        """
+        def wrap(coro):
+            callback = ModalCallback(custom_id, coro)
+            self.add_callback(callback)
+            return callback
+        return wrap
