@@ -13,9 +13,10 @@ class InteractionContext(Interaction):
         self.response = asyncio.Future()
         self.deferred = False
         self.logger = resp["logger"]
+        self.options = {}
 
     def defer(self, ephemeral: bool = False, update_message: bool = False):
-        if self.type.application_command:
+        if self.type.application_command or self.type.modal_submit:
             if update_message:
                 self.logger.warning("update_message is only for message component. Ignoring update_message param.")
             callback_type = InteractionCallbackType.DEFERRED_CHANNEL_MESSAGE_WITH_SOURCE
@@ -40,6 +41,8 @@ class InteractionContext(Interaction):
              allowed_mentions: typing.Union[AllowedMentions, dict] = None,
              components: typing.List[typing.Union[dict, Component]] = None,
              choices: typing.Optional[typing.List[typing.Union[dict, ApplicationCommandOptionChoice]]] = None,
+             custom_id: str = None,
+             title: str = None,
              ephemeral: bool = False,
              update_message: bool = False):
         if self.type.application_command:
@@ -53,14 +56,16 @@ class InteractionContext(Interaction):
             if file or files:
                 self.logger.warning("file and files are not supported on initial response. Ignoring file or files param.")
             callback_type = InteractionCallbackType.APPLICATION_COMMAND_AUTOCOMPLETE_RESULT if self.type.application_command_autocomplete else \
-                InteractionCallbackType.UPDATE_MESSAGE if update_message else InteractionCallbackType.CHANNEL_MESSAGE_WITH_SOURCE
+                InteractionCallbackType.UPDATE_MESSAGE if update_message else InteractionCallbackType.MODAL if custom_id else InteractionCallbackType.CHANNEL_MESSAGE_WITH_SOURCE
             data = InteractionApplicationCommandCallbackData(tts=tts,
                                                              content=content,
                                                              embeds=embeds,
                                                              allowed_mentions=allowed_mentions,
                                                              flags=64 if ephemeral else None,
                                                              components=components,
-                                                             choices=choices)
+                                                             choices=choices,
+                                                             custom_id=custom_id,
+                                                             title=title)
             resp = InteractionResponse(callback_type, data)
             self.deferred = True
             return self.create_response(resp)
@@ -83,6 +88,17 @@ class InteractionContext(Interaction):
         else:
             return await super().create_response(interaction_response)
 
+    def get_value(self, custom_id: str):
+        if not self.type.modal_submit:
+            raise AttributeError("this is only allowed for modal submit")
+        comps = []
+        for x in self.data.components:
+            comps.extend(x.components)
+        resp = [y for y in comps if y.custom_id == custom_id]
+        if resp:
+            return resp[0].value
+        raise KeyError(custom_id)
+
     @classmethod
     def from_interaction(cls, interaction: Interaction, logger):
         resp = interaction.raw
